dofile("./Config/export/utils.lua")

LoLibData=
{
	UpdatePeriod = 5, --In seconds
	RemoteAddress = "192.168.1.255",
	RemotePort = 10311,
	Debug = false,
	
	-- --------------------------------------------------------------------
	-- Fonction de dmarrage du script
	-- --------------------------------------------------------------------
	Start=function(self)
		dofile "lua.lua"	
		socket = require("socket")

		self.Objects={}

		self.NextUpdateTime=0
		
		self.XmlHeader = "<?xml version=\"1.0\" encoding=\"utf-16\"?>"

		-- Initialisation du socket de communication
		self.SockCom = socket.udp()
		self.SockCom:setpeername(self.RemoteAddress, self.RemotePort)
		self.SockCom:settimeout(0.000001) -- Permet de ne pas bloquer la lecture
		
		--Initialisation du journal (log)
		if self.Debug then
			self.LogFile = io.open("./Temp/LONET-Debug.txt","w")
			if self.LogFile then
				self.LogFile:write(self.XmlHeader);
				self.LogFile:write("<ArrayOfBasicEvent xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">");
			end
		end
	end,

	-- --------------------------------------------------------------------
	-- Fonction de rafraichissement du script
	-- --------------------------------------------------------------------
	Update=function(self)
		local CurrentTime=LoGetModelTime();

		local ShouldUpdateLog=false;

		if CurrentTime>self.NextUpdateTime then -- Si on a atteind le moment de rafraichissement
			self.NextUpdateTime=CurrentTime+self.UpdatePeriod;
		else
			return
		end
		
		-- Vrification des messages d'entres
		incomingData = self.SockCom:receive()
		if incomingData then
			LoSetCommand(incomingData)
		end
		
		isMyPlaneAlive = LoGetTrueAirSpeed()
		
		stackMission = newStack()
	
		addString(stackMission, "<mission>")
		addString(stackMission, "<modelTime>" .. LoGetModelTime() .. "</modelTime>")
		addString(stackMission, "<missionStartTime>" .. LoGetMissionStartTime() .. "</missionStartTime>")
		if isMyPlaneAlive then
			addString(stackMission, "<basicAtmospherePressure>" .. LoGetBasicAtmospherePressure() .. "</basicAtmospherePressure>")
		end
		addString(stackMission, "</mission>")

		stackMyPlane = newStack()
		
		
		if isMyPlaneAlive then
		
		addString(stackMyPlane, "<playerPlane>")
		writeObjectPropertyInStack(stackMyPlane, LoGetObjectById(LoGetPlayerPlaneId()), LoGetPlayerPlaneId())		
		addString(stackMyPlane, "<pilotName>" .. LoGetPilotName() .. "</pilotName>")
		addString(stackMyPlane, "<indicatedAirSpeed>" .. LoGetIndicatedAirSpeed() .. "</indicatedAirSpeed>")
		addString(stackMyPlane, "<trueAirSpeed>" .. LoGetTrueAirSpeed() .. "</trueAirSpeed>")
		addString(stackMyPlane, "<altitudeAboveSeaLevel>" .. LoGetAltitudeAboveSeaLevel() .. "</altitudeAboveSeaLevel>")
		addString(stackMyPlane, "<altitudeAboveGroundLevel>" .. LoGetAltitudeAboveGroundLevel() .. "</altitudeAboveGroundLevel>")
		addString(stackMyPlane, "<angleOfAttack>" .. LoGetAngleOfAttack() .. "</angleOfAttack>")
		addString(stackMyPlane, "<accelerationUnits>" .. LoGetAccelerationUnits() .. "</accelerationUnits>")
		addString(stackMyPlane, "<verticalVelocity>" .. LoGetVerticalVelocity() .. "</verticalVelocity>")
		addString(stackMyPlane, "<machNumber>" .. LoGetMachNumber() .. "</machNumber>")
		local pitch, bank, yaw = LoGetADIPitchBankYaw()
		addString(stackMyPlane, "<pitch>" .. pitch .. "</pitch>")
		addString(stackMyPlane, "<bank>" .. bank .. "</bank>")
		addString(stackMyPlane, "<yaw>" .. yaw .. "</yaw>")
		addString(stackMyPlane, "<magneticYaw>" .. LoGetMagneticYaw() .. "</magneticYaw>")
		addString(stackMyPlane, "<glideDeviation>" .. LoGetGlideDeviation() .. "</glideDeviation>")
		addString(stackMyPlane, "<sideDeviation>" .. LoGetSideDeviation() .. "</sideDeviation>")
		addString(stackMyPlane, "<slipBallPosition>" .. LoGetSlipBallPosition() .. "</slipBallPosition>")
		addString(stackMyPlane, "<controlPanelHSI>")
			addString(stackMyPlane, "<ADF>" .. LoGetControlPanel_HSI().ADF .. "</ADF>")
			addString(stackMyPlane, "<RMI>" .. LoGetControlPanel_HSI().RMI .. "</RMI>")
			addString(stackMyPlane, "<compass>" .. LoGetControlPanel_HSI().Compass .. "</compass>")
		addString(stackMyPlane, "</controlPanelHSI>")
		
		-- On s'occupe de l'ensemble propulsion de l'avion
		addString(stackMyPlane, "<engineInfo>")
			addString(stackMyPlane, "<RPMLeft>" .. LoGetEngineInfo().RPM.left .. "</RPMLeft>")
			addString(stackMyPlane, "<RPMRight>" .. LoGetEngineInfo().RPM.right .. "</RPMRight>")
			addString(stackMyPlane, "<temperatureLeft>" .. LoGetEngineInfo().Temperature.left .. "</temperatureLeft>")
			addString(stackMyPlane, "<temperatureRight>" .. LoGetEngineInfo().Temperature.right .. "</temperatureRight>")
			addString(stackMyPlane, "<hydraulicPressureLeft>" .. LoGetEngineInfo().HydraulicPressure.left .. "</hydraulicPressureLeft>")
			addString(stackMyPlane, "<hydraulicPressureRight>" .. LoGetEngineInfo().HydraulicPressure.right .. "</hydraulicPressureRight>")
			addString(stackMyPlane, "<fuelInternal>" .. LoGetEngineInfo().fuel_internal .. "</fuelInternal>")
			addString(stackMyPlane, "<fuelExternal>" .. LoGetEngineInfo().fuel_external .. "</fuelExternal>")
		addString(stackMyPlane, "</engineInfo>")
		
		
		
		addString(stackMyPlane, "<mcpStates>")
		for strKey, boolValue in pairs(LoGetMCPState()) do
			if(boolValue) then
				addString(stackMyPlane, "<McpState>" .. strKey .. "</McpState>")
			end
		end		
		addString(stackMyPlane, "</mcpStates>")
	
		-- On s'occupe maintenant du systme de gestion de la navigation (autopilote, etc ...).
		addString(stackMyPlane, "<navigationInfo>")
			addString(stackMyPlane, "<masterMode>" .. LoGetNavigationInfo().SystemMode.master .. "</masterMode>"	)
			addString(stackMyPlane, "<subMode>" .. LoGetNavigationInfo().SystemMode.submode .. "</subMode>")
			addString(stackMyPlane, "<requiredRoll>" .. LoGetNavigationInfo().Requirements.roll .. "</requiredRoll>")
			addString(stackMyPlane, "<requiredPitch>" .. LoGetNavigationInfo().Requirements.pitch .. "</requiredPitch>")
			addString(stackMyPlane, "<requiredSpeed>" .. LoGetNavigationInfo().Requirements.speed .. "</requiredSpeed>")
			addString(stackMyPlane, "<requiredVerticalSpeed>" .. LoGetNavigationInfo().Requirements.vertical_speed .. "</requiredVerticalSpeed>")
			addString(stackMyPlane, "<requiredAltitude>" .. LoGetNavigationInfo().Requirements.altitude .. "</requiredAltitude>")
			addString(stackMyPlane, "<acsMode>" .. LoGetNavigationInfo().ACS.mode .. "</acsMode>")
			local autothrust = ""
			if LoGetNavigationInfo().ACS.autothrust then
				autothrust = "true"
			else
				autothrust = "false"
			end
			addString(stackMyPlane, "<isAutothrust>" .. autothrust .. "</isAutothrust>")
		addString(stackMyPlane, "</navigationInfo>")
		
		addString(stackMyPlane, "<route>")
			addString(stackMyPlane, "<gotoPoint>")
				addString(stackMyPlane, "<pointNum>" .. LoGetRoute().goto_point.this_point_num .. "</pointNum>")
				addString(stackMyPlane, "<worldPoint>")
					addString(stackMyPlane, "<x>" .. LoGetRoute().goto_point.world_point.x .. "</x>")
					addString(stackMyPlane, "<y>" .. LoGetRoute().goto_point.world_point.y .. "</y>")
					addString(stackMyPlane, "<z>" .. LoGetRoute().goto_point.world_point.z .. "</z>")
				addString(stackMyPlane, "</worldPoint>")
				addString(stackMyPlane, "<speedRequired>" .. LoGetRoute().goto_point.speed_req .. "</speedRequired>")
				addString(stackMyPlane, "<estimatedTime>" .. LoGetRoute().goto_point.estimated_time .. "</estimatedTime>")
				addString(stackMyPlane, "<nextPointNum>" .. LoGetRoute().goto_point.next_point_num .. "</nextPointNum>")
			addString(stackMyPlane, "</gotoPoint>")
			addString(stackMyPlane, "<route>")
				for index,waypoint in pairs(LoGetRoute().route) do
					addString(stackMyPlane, "<BasicWaypoint>")
						addString(stackMyPlane, "<pointNum>" .. waypoint.this_point_num .. "</pointNum>")
						addString(stackMyPlane, "<worldPoint>")
							addString(stackMyPlane, "<x>" .. waypoint.world_point.x .. "</x>")
							addString(stackMyPlane, "<y>" .. waypoint.world_point.y .. "</y>")
							addString(stackMyPlane, "<z>" .. waypoint.world_point.z .. "</z>")
						addString(stackMyPlane, "</worldPoint>")
						addString(stackMyPlane, "<speedRequired>" .. waypoint.speed_req .. "</speedRequired>")
						addString(stackMyPlane, "<estimatedTime>" .. waypoint.estimated_time .. "</estimatedTime>")
						addString(stackMyPlane, "<nextPointNum>" .. waypoint.next_point_num .. "</nextPointNum>")
					addString(stackMyPlane, "</BasicWaypoint>")
				end
			addString(stackMyPlane, "</route>")
		addString(stackMyPlane, "</route>")


		addString(stackMyPlane, "<targets>")
		-- Boucle pour rcuprer tous les targets
		for key,target in pairs(LoGetTargetInformation()) do
			addString(stackMyPlane, "<BasicTargetInfo>")
				writeObjectPropertyInStack(stackMyPlane, LoGetObjectById(target.ID), target.ID)		
				addString(stackMyPlane, "<classification>")
					addString(stackMyPlane, "<level1>" .. target.type.level1 .. "</level1>")
					addString(stackMyPlane, "<level2>" .. target.type.level2 .. "</level2>")
					addString(stackMyPlane, "<level3>" .. target.type.level3 .. "</level3>")
					addString(stackMyPlane, "<level4>" .. target.type.level4 .. "</level4>")
				addString(stackMyPlane, "</classification>")
				addString(stackMyPlane, "<orientationX>")
					addString(stackMyPlane, "<x>" .. target.position.x.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.position.x.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.position.x.z .. "</z>")
				addString(stackMyPlane, "</orientationX>")
				addString(stackMyPlane, "<orientationY>")
					addString(stackMyPlane, "<x>" .. target.position.y.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.position.y.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.position.y.z .. "</z>")
				addString(stackMyPlane, "</orientationY>")
				addString(stackMyPlane, "<orientationZ>")
					addString(stackMyPlane, "<x>" .. target.position.z.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.position.z.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.position.z.z .. "</z>")
				addString(stackMyPlane, "</orientationZ>")
				addString(stackMyPlane, "<centerPosition>")
					addString(stackMyPlane, "<x>" .. target.position.p.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.position.p.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.position.p.z .. "</z>")
				addString(stackMyPlane, "</centerPosition>")
				addString(stackMyPlane, "<velocity>")
					addString(stackMyPlane, "<x>" .. target.velocity.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.velocity.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.velocity.z .. "</z>")
				addString(stackMyPlane, "</velocity>")
				addString(stackMyPlane, "<distance>" .. target.distance .. "</distance>")
				addString(stackMyPlane, "<convergenceVelocity>" .. target.convergence_velocity .. "</convergenceVelocity>")
				addString(stackMyPlane, "<mach>" .. target.mach .. "</mach>")
				addString(stackMyPlane, "<angleAspect>" .. target.delta_psi .. "</angleAspect>")
				addString(stackMyPlane, "<fim>" .. target.fim .. "</fim>")
				addString(stackMyPlane, "<fin>" .. target.fin .. "</fin>")
				addString(stackMyPlane, "<trackingType>" .. getGoodTrackingType(target.flags) .. "</trackingType>")
				addString(stackMyPlane, "<reflection>" .. target.reflection .. "</reflection>")
				addString(stackMyPlane, "<course>" .. target.course .. "</course>")
				addString(stackMyPlane, "<isJamming>" .. target.isjamming .. "</isJamming>")
				addString(stackMyPlane, "<startOfLock>" .. target.start_of_lock .. "</startOfLock>")
				addString(stackMyPlane, "<forces>")
					addString(stackMyPlane, "<x>" .. target.forces.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.forces.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.forces.z .. "</z>")
				addString(stackMyPlane, "</forces>")
				addString(stackMyPlane, "<radarUpdatesNumber>" .. target.updates_number .. "</radarUpdatesNumber>")
			addString(stackMyPlane, "</BasicTargetInfo>")
		end
		addString(stackMyPlane, "</targets>")
		
		
		addString(stackMyPlane, "<lockedTargets>")
		-- Boucle pour rcuprer tous les targets
		for key,target in pairs(LoGetLockedTargetInformation()) do
			addString(stackMyPlane, "<BasicTargetInfo>")
				writeObjectPropertyInStack(stackMyPlane, LoGetObjectById(target.ID), target.ID)	
				addString(stackMyPlane, "<classification>")
					addString(stackMyPlane, "<level1>" .. target.type.level1 .. "</level1>")
					addString(stackMyPlane, "<level2>" .. target.type.level2 .. "</level2>")
					addString(stackMyPlane, "<level3>" .. target.type.level3 .. "</level3>")
					addString(stackMyPlane, "<level4>" .. target.type.level4 .. "</level4>")
				addString(stackMyPlane, "</classification>")
				addString(stackMyPlane, "<orientationX>")
					addString(stackMyPlane, "<x>" .. target.position.x.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.position.x.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.position.x.z .. "</z>")
				addString(stackMyPlane, "</orientationX>")
				addString(stackMyPlane, "<orientationY>")
					addString(stackMyPlane, "<x>" .. target.position.y.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.position.y.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.position.y.z .. "</z>")
				addString(stackMyPlane, "</orientationY>")
				addString(stackMyPlane, "<orientationZ>")
					addString(stackMyPlane, "<x>" .. target.position.z.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.position.z.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.position.z.z .. "</z>")
				addString(stackMyPlane, "</orientationZ>")
				addString(stackMyPlane, "<centerPosition>")
					addString(stackMyPlane, "<x>" .. target.position.p.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.position.p.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.position.p.z .. "</z>")
				addString(stackMyPlane, "</centerPosition>")
				addString(stackMyPlane, "<velocity>")
					addString(stackMyPlane, "<x>" .. target.velocity.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.velocity.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.velocity.z .. "</z>")
				addString(stackMyPlane, "</velocity>")
				addString(stackMyPlane, "<distance>" .. target.distance .. "</distance>")
				addString(stackMyPlane, "<convergenceVelocity>" .. target.convergence_velocity .. "</convergenceVelocity>")
				addString(stackMyPlane, "<mach>" .. target.mach .. "</mach>")
				addString(stackMyPlane, "<angleAspect>" .. target.delta_psi .. "</angleAspect>")
				addString(stackMyPlane, "<fim>" .. target.fim .. "</fim>")
				addString(stackMyPlane, "<fin>" .. target.fin .. "</fin>")
				addString(stackMyPlane, "<trackingType>" .. getGoodTrackingType(target.flags) .. "</trackingType>")
				addString(stackMyPlane, "<reflection>" .. target.reflection .. "</reflection>")
				addString(stackMyPlane, "<course>" .. target.course .. "</course>")
				addString(stackMyPlane, "<isJamming>" .. target.isjamming .. "</isJamming>")
				addString(stackMyPlane, "<startOfLock>" .. target.start_of_lock .. "</startOfLock>")
				addString(stackMyPlane, "<forces>")
					addString(stackMyPlane, "<x>" .. target.forces.x .. "</x>")
					addString(stackMyPlane, "<y>" .. target.forces.y .. "</y>")
					addString(stackMyPlane, "<z>" .. target.forces.z .. "</z>")
				addString(stackMyPlane, "</forces>")
				addString(stackMyPlane, "<radarUpdatesNumber>" .. target.updates_number .. "</radarUpdatesNumber>")
			addString(stackMyPlane, "</BasicTargetInfo>")
		end
		addString(stackMyPlane, "</lockedTargets>")
			
	
		local tws = LoGetTWSInfo()
		addString(stackMyPlane, "<twsInfo>")
			local strTws = ""
			if tws.Mode == 0 then
				strTws = "All"
			elseif tws.Mode == 1 then
				strTws = "LockOnly"
			elseif tws.Mode == 2 then
				strTws = "LaunchOnly"
			end
			addString(stackMyPlane, "<mode>" .. strTws .. "</mode>")
			addString(stackMyPlane, "<emitters>")
			-- Boucle pour rcuprer tous les emetteurs
			for key,emitter in pairs(tws.Emitters) do
				addString(stackMyPlane, "<BasicEmitter>")
					writeObjectPropertyInStack(stackMyPlane, LoGetObjectById(emitter.ID), emitter.ID)
					addString(stackMyPlane, "<classification>")
						addString(stackMyPlane, "<level1>" .. emitter.Type.level1 .. "</level1>")
						addString(stackMyPlane, "<level2>" .. emitter.Type.level2 .. "</level2>")
						addString(stackMyPlane, "<level3>" .. emitter.Type.level3 .. "</level3>")
						addString(stackMyPlane, "<level4>" .. emitter.Type.level4 .. "</level4>")
					addString(stackMyPlane, "</classification>")
					addString(stackMyPlane, "<power>" .. emitter.Power .. "</power>")
					addString(stackMyPlane, "<priority>" .. emitter.Priority .. "</priority>")
					addString(stackMyPlane, "<signalType>" .. getGoodSignalType(emitter.SignalType) .. "</signalType>")
				addString(stackMyPlane, "</BasicEmitter>")
			end
			addString(stackMyPlane, "</emitters>")
		addString(stackMyPlane, "</twsInfo>")
	
	
		local payload = LoGetPayloadInfo()
		addString(stackMyPlane, "<payloadInfo>")
			addString(stackMyPlane, "<currentStation>" .. payload.CurrentStation .. "</currentStation>")
			addString(stackMyPlane, "<stations>")
			-- Boucle pour rcuprer tous les emports
			for key,station in pairs(payload.Stations) do
				addString(stackMyPlane, "<station>")
					addString(stackMyPlane, "<position>" .. key .. "</position>")
					addString(stackMyPlane, "<container>")
						if (station.container) then addString(stackMyPlane, "true") else addString(stackMyPlane, "false") end
					addString(stackMyPlane, "</container>")
					addString(stackMyPlane, "<classification>")
						addString(stackMyPlane, "<level1>" .. station.weapon.level1 .. "</level1>")
						addString(stackMyPlane, "<level2>" .. station.weapon.level2 .. "</level2>")
						addString(stackMyPlane, "<level3>" .. station.weapon.level3 .. "</level3>")
						addString(stackMyPlane, "<level4>" .. station.weapon.level4 .. "</level4>")
					addString(stackMyPlane, "</classification>")
					addString(stackMyPlane, "<count>" .. station.count .. "</count>")
				addString(stackMyPlane, "</station>")
			end
			addString(stackMyPlane, "</stations>")
		addString(stackMyPlane, "</payloadInfo>")	
		addString(stackMyPlane, "</playerPlane>")
		
		end --Enf if my plane is alive
		
		stackObjects = newStack()
		
		addString(stackObjects, "<objects>")
		local objects = LoGetWorldObjects()
		if objects then
			for key,object in pairs(objects) do
				addString(stackObjects, "<BasicObject>")
					writeObjectPropertyInStack(stackObjects, object, key)
				addString(stackObjects, "</BasicObject>")
			end
		end
		addString(stackObjects, "</objects>")
			
		xmlMission = table.concat(stackMission)
		xmlPlayerPlane = table.concat(stackMyPlane)
		xmlObjects = table.concat(stackObjects)
		
		stackAllData = newStack()
		addString(stackAllData, xmlMission)
		addString(stackAllData, xmlPlayerPlane)
		addString(stackAllData, xmlObjects)
		
		xmlAllData = table.concat(stackAllData)
		
		stackLoEventForNetwork = newStack()
		addString(stackLoEventForNetwork, self.XmlHeader)
		addString(stackLoEventForNetwork, "<BasicEvent xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">")
		addString(stackLoEventForNetwork, xmlAllData)
		addString(stackLoEventForNetwork, "</BasicEvent>")
		
		xmlLoEventForNetwork = table.concat(stackLoEventForNetwork)
		
		stackLoEventForLog = newStack()
		addString(stackLoEventForLog, "<BasicEvent>")
		addString(stackLoEventForLog, xmlAllData)
		addString(stackLoEventForLog, "</BasicEvent>")
		
		xmlLoEventForLog = table.concat(stackLoEventForLog)

		if self.Debug then
			self.LogFile:write(xmlLoEventForLog)
			self.LogFile:write("\n")
		end
			
		if self.SockCom then
			self.SockCom:send(xmlLoEventForNetwork)
		end
	end,
		
	-- --------------------------------------------------------------------
	-- Fonction d'arrt du script
	-- --------------------------------------------------------------------
	Stop=function(self)
		if self.LogFile then
			self.LogFile:write("</ArrayOfBasicEvent>")
			io.close(self.LogFile)
		end
	end,
}

-- --------------------------------------------------------------------------------------------

-- Works once just before mission start.
do
	local PrevLuaExportStart=LuaExportStart;

	LuaExportStart=function()
		LoLibData:Start();

		if PrevLuaExportStart then
			PrevLuaExportStart();
		end
	end
end

-- Works just after every simulation frame.
do
	local PrevLuaExportAfterNextFrame=LuaExportAfterNextFrame;

	LuaExportAfterNextFrame=function()
		LoLibData:Update();

		if PrevLuaExportAfterNextFrame then
			PrevLuaExportAfterNextFrame();
		end
	end
end

-- Works once just after mission stop.
do
	local PrevLuaExportStop=LuaExportStop;

	LuaExportStop=function()
		LoLibData:Stop();

		if PrevLuaExportStop then
			PrevLuaExportStop();
		end
	end
end
